/* Test program to embed python into C++ and call back via SWIG */

#include <Python.h>
#include <stdio.h>
#include "example.h"

int main(int argc, char **argv)
{
    char *to_import = "mytest";
    char *to_run = "main";
    PyObject *pModule, *pDict, *pFunc;
    PyObject *pArgs, *pValue;
    int rc = 1;			// default is error

    Py_Initialize();

    // Allow the python code to read the swig symbols
    // swig_add_builtin();
    // PyImport_Inittab = _SwigImport_Inittab;	

    // Bodgy way of setting the path!
    PyRun_SimpleString("import sys\nsys.path.insert(0, '.')\n");

    // Import the module
    pModule = PyImport_ImportModule(to_import);
    if (pModule == NULL)
    {
        PyErr_Print();
        fprintf(stderr, "Failed to load \"%s\"\n", to_import);
        goto error1;
    }

    pDict = PyModule_GetDict(pModule);
    /* pDict is a borrowed reference */
    if (pDict == 0)
    {
	PyErr_Print();
	fprintf(stderr,"Couldn't find dict from module\n");
        goto error2;
    }
    
    pFunc = PyDict_GetItemString(pDict, to_run);
    /* pFunc: Borrowed reference */
    if (pFunc == 0)
    {
	PyErr_Print();
	fprintf(stderr,"Couldn't find '%s'\n", to_run);
        goto error2;
    }

    My_variable = 1.2345;
    pArgs = Py_BuildValue("(s)", "sausage");
    pValue = PyObject_CallObject(pFunc, pArgs);
    Py_DECREF(pArgs);
    if (pValue == 0)
    {
	PyErr_Print();
	fprintf(stderr,"Call failed\n");
	goto error2;
    }

    printf("Result of call: ");
    PyObject_Print(pValue, stdout, 0);
    printf("\n");
    Py_DECREF(pValue);

    /* pDict and pFunc are borrowed and must not be Py_DECREF-ed */
    rc = 0;			// no error
error2:;
    Py_DECREF(pModule);
error1:;
    Py_Finalize();
    return rc;
}
